from django import template
from django.conf import settings
from django.utils.module_loading import import_string

from template_shortcuts.nodes import CSSTag, JSTag

PROVIDER = import_string(getattr(settings, "TEMPLATE_CDN_PROVIDER", None))()


def parse_tag(parser, token):
    try:
        # split_contents() knows not to split quoted strings.
        tag_name, version = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires a single argument" %
            token.contents.split()[0])
    if not (version[0] == version[-1] and version[0] in ('"', "'")):
        raise template.TemplateSyntaxError(
            "%r tag's argument should be in quotes" % tag_name)
    return tag_name, version[1:-1]


def javascript_tag(parser, token):
    tag_name, version = parse_tag(parser, token)
    url_builder = getattr(PROVIDER, tag_name)
    if url_builder:
        url = url_builder(version)
        return JSTag(url)
    raise template.TemplateSyntaxError(
        "Unsupported library '%s'" % tag_name)


def css_tag(parser, token):
    tag_name, version = parse_tag(parser, token)
    url_builder = getattr(PROVIDER, tag_name)
    if url_builder:
        url = url_builder(version)
        return CSSTag(url)
    raise template.TemplateSyntaxError(
        "Unsupported library '%s'" % tag_name)

register = template.Library()
JS_TAGS = ["angular", "chrome_frame", "dojo", "ext_core",
           "jquery", "jquery_ui", "mootools", "modernizr",
           "prototype", "scriptaculos", "webfont"]

for tag in JS_TAGS:
    register.tag(tag, javascript_tag)



from .provider import CDNProvider


class Google(CDNProvider):
    base_url = "//ajax.googleapis.com/ajax/libs/%s/%s/%s"

    def angular(self, version):
        return self._build_js_url("angularjs", version, "angular")

    def chrome_frame(self, version):
        return self._build_js_url("chrome-frame", version, "CFInstall")

    def jquery(self, version):
        return self._build_js_url("jquery", version)

    def ext_core(self, version):
        return self._build_js_url("ext-core", version)

    def jquery_ui(self, version):
        return self._build_js_url("jqueryui", version, "jquery-ui")

    def mootools(self, version):
        return self._build_js_url("mootools", version,
                                  "mootools-yui-compressed.js")

    def prototype(self, version):
        return self._build_js_url("prototype", version)

    def scriptaculos(self, version):
        return self._build_js_url("scriptaculos", version)

    def swfobject(self, version):
        return self._build_js_url("swfobject", version)

    def webfont(self, version):
        return self._build_js_url("webfont", version)



from .provider import CDNProvider


class CDNJS(CDNProvider):
    base_url = "//cdnjs.cloudflare.com/ajax/libs/%s/%s/%s"

    def angular(self, version):
        return self._build_js_url("angular.js", version, "angular")

    def chrome_frame(self, version):
        return self._build_js_url("chrome-frame", version, "CFInstall")

    def modernizr(self, version):
        return self._build_js_url("modernizr", version)

    def jquery(self, version):
        return self._build_js_url("jquery", version)

    def ext_core(self, version):
        return self._build_js_url("ext-core", version)

    def jquery_ui(self, version):
        return self._build_js_url("jqueryui", version, "jquery-ui")

    def mootools(self, version):
        return self._build_js_url("mootools", version,
                                  "mootools-core-full-compat-yc")



import os.path
from setuptools import setup, find_packages

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

setup(
    name="django-template-shortcuts",
    version="0.2.2",
    packages=find_packages(),
    author="Andrei Coman",
    author_email="comandrei@gmail.com",
    install_requires=["django>=1.7"],
    url="https://github.com/comandrei/django-template-shortcuts/",
    license='BSD',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    description='Reusable Django templatetags',
    long_description=README
)



class CDNProvider(object):
    base_url = NotImplemented

    def _build_url(self, lib_name, version, file_name=None):
        if file_name is None:
            file_name = lib_name
        return self.base_url % (lib_name, version, file_name)

    def _build_js_url(self, lib_name, version, file_name=None):
        url = self._build_url(lib_name, version, file_name)
        return "%s.min.js" % url

    def _build_css_url(self, lib_name, version, file_name=None):
        url = self._build_url(lib_name, version, file_name)
        return "%s.css" % url

    def angular(self, version):
        raise NotImplementedError()

    def jquery(self, version):
        raise NotImplementedError()

    def modernizr(self, version):
        raise NotImplementedError()

    def chrome_frame(self, version):
        raise NotImplementedError()

    def dojo(self, version):
        raise NotImplementedError()

    def ext_core(self, version):
        raise NotImplementedError()

    def jquery_ui(self, version):
        raise NotImplementedError()

    def mootools(self, version):
        raise NotImplementedError()

    def prototype(self, version):
        raise NotImplementedError()

    def scriptaculos(self, version):
        raise NotImplementedError()

    def swfobject(self, version):
        raise NotImplementedError()

    def webfont(self, version):
        raise NotImplementedError()



from django import template


class CSSTag(template.Node):

    def __init__(self, url):
        self.url = url

    def render(self, context):
        return '<link href="%s" type="text/css" rel="stylesheet" />' % self.url


class JSTag(template.Node):

    def __init__(self, url):
        self.url = url

    def render(self, context):
        return '<script type="text/javascript" src="%s"></script>' % self.url
